@@ -16,8 +16,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 /**
@@ -28,7 +28,7 @@
  * <refsect2>
  * <title>Example launch line</title>
  * |[
- * gst-launch -v aravissrc ! video/x-raw-yuv,width=512,height=512,framerate=25/1 ! autovideosink
+ * gst-launch -v aravissrc ! video/x-raw,format=UYVY,width=512,height=512,framerate=25/1 ! autovideosink
  * ]|
  * </refsect2>
  */
@@ -56,10 +56,11 @@ enum
   PROP_H_BINNING,
   PROP_V_BINNING,
   PROP_OFFSET_X,
-  PROP_OFFSET_Y
+  PROP_OFFSET_Y,
+  PROP_PACKET_RESEND
 };
 
-GST_BOILERPLATE (GstAravis, gst_aravis, GstPushSrc, GST_TYPE_PUSH_SRC);
+G_DEFINE_TYPE (GstAravis, gst_aravis, GST_TYPE_PUSH_SRC);
 
 static GstStaticPadTemplate aravis_src_template = GST_STATIC_PAD_TEMPLATE ("src",
 									   GST_PAD_SRC,
@@ -123,7 +124,7 @@ gst_aravis_get_all_camera_caps (GstAravis *gst_aravis)
 }
 
 static GstCaps *
-gst_aravis_get_caps (GstBaseSrc * src)
+gst_aravis_get_caps (GstBaseSrc * src, GstCaps * filter)
 {
 	GstAravis* gst_aravis = GST_ARAVIS(src);
 	GstCaps *caps;
@@ -145,11 +146,10 @@ gst_aravis_set_caps (GstBaseSrc *src, GstCaps *caps)
 	GstStructure *structure;
 	ArvPixelFormat pixel_format;
 	int height, width;
-	int bpp, depth;
 	const GValue *frame_rate;
 	const char *caps_string;
+	const char *format_string;
 	unsigned int i;
-	guint32 fourcc;
 
 	GST_LOG_OBJECT (gst_aravis, "Requested caps = %" GST_PTR_FORMAT, caps);
 
@@ -163,20 +163,9 @@ gst_aravis_set_caps (GstBaseSrc *src, GstCaps *caps)
 	gst_structure_get_int (structure, "width", &width);
 	gst_structure_get_int (structure, "height", &height);
 	frame_rate = gst_structure_get_value (structure, "framerate");
-	gst_structure_get_int (structure, "bpp", &bpp);
-	gst_structure_get_int (structure, "depth", &depth);
+	format_string = gst_structure_get_string (structure, "format");
 
-	if (gst_structure_get_field_type (structure, "format") == G_TYPE_STRING) {
-		const char *string;
-
-	       	string = gst_structure_get_string (structure, "format");
-		fourcc = GST_STR_FOURCC (string);
-	} else if (gst_structure_get_field_type (structure, "format") == GST_TYPE_FOURCC) {
-		gst_structure_get_fourcc (structure, "format", &fourcc);
-	} else
-		fourcc = 0;
-
-	pixel_format = arv_pixel_format_from_gst_caps (gst_structure_get_name (structure), bpp, depth, fourcc);
+	pixel_format = arv_pixel_format_from_gst_caps (gst_structure_get_name (structure), format_string);
 
 	arv_camera_set_region (gst_aravis->camera, gst_aravis->offset_x, gst_aravis->offset_y, width, height);
 	arv_camera_set_binning (gst_aravis->camera, gst_aravis->h_binning, gst_aravis->v_binning);
@@ -199,25 +188,25 @@ gst_aravis_set_caps (GstBaseSrc *src, GstCaps *caps)
 	} else
 		gst_aravis->buffer_timeout_us = GST_ARAVIS_BUFFER_TIMEOUT_DEFAULT;
 
-	GST_DEBUG_OBJECT (gst_aravis, "Buffer timeout = %Ld µs", gst_aravis->buffer_timeout_us);
+	GST_DEBUG_OBJECT (gst_aravis, "Buffer timeout = %" G_GUINT64_FORMAT " µs", gst_aravis->buffer_timeout_us);
 
 	GST_DEBUG_OBJECT (gst_aravis, "Actual frame rate = %g Hz", arv_camera_get_frame_rate (gst_aravis->camera));
 
 	if(gst_aravis->gain_auto) {
 		arv_camera_set_gain_auto (gst_aravis->camera, ARV_AUTO_CONTINUOUS);
-		GST_DEBUG_OBJECT (gst_aravis, "Auto Gain = continuous", gst_aravis->gain_auto);
+		GST_DEBUG_OBJECT (gst_aravis, "Auto Gain = continuous");
 	} else {
 		if (gst_aravis->gain >= 0) {
-			GST_DEBUG_OBJECT (gst_aravis, "Gain = %d", gst_aravis->gain);
+			GST_DEBUG_OBJECT (gst_aravis, "Gain = %g", gst_aravis->gain);
 			arv_camera_set_gain_auto (gst_aravis->camera, ARV_AUTO_OFF);
 			arv_camera_set_gain (gst_aravis->camera, gst_aravis->gain);
 		}
-		GST_DEBUG_OBJECT (gst_aravis, "Actual gain = %d", arv_camera_get_gain (gst_aravis->camera));
+		GST_DEBUG_OBJECT (gst_aravis, "Actual gain = %g", arv_camera_get_gain (gst_aravis->camera));
 	}
 
 	if(gst_aravis->exposure_auto) {
 		arv_camera_set_exposure_time_auto (gst_aravis->camera, ARV_AUTO_CONTINUOUS);
-		GST_DEBUG_OBJECT (gst_aravis, "Auto Exposure = contiuous", gst_aravis->exposure_auto);
+		GST_DEBUG_OBJECT (gst_aravis, "Auto Exposure = continuous");
 	} else {
 		if (gst_aravis->exposure_time_us > 0.0) {
 			GST_DEBUG_OBJECT (gst_aravis, "Exposure = %g µs", gst_aravis->exposure_time_us);
@@ -254,6 +243,11 @@ gst_aravis_set_caps (GstBaseSrc *src, GstCaps *caps)
 	gst_aravis->payload = arv_camera_get_payload (gst_aravis->camera);
 	gst_aravis->stream = arv_camera_create_stream (gst_aravis->camera, NULL, NULL);
 
+	if (ARV_IS_GV_STREAM (gst_aravis->stream) && gst_aravis->packet_resend)
+                g_object_set (gst_aravis->stream, "packet-resend", ARV_GV_STREAM_PACKET_RESEND_ALWAYS, NULL);
+        else
+                g_object_set (gst_aravis->stream, "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL);
+
 	for (i = 0; i < GST_ARAVIS_N_BUFFERS; i++)
 		arv_stream_push_buffer (gst_aravis->stream,
 					arv_buffer_new (gst_aravis->payload, NULL));
@@ -331,7 +325,7 @@ gst_aravis_get_times (GstBaseSrc * basesrc, GstBuffer * buffer,
 		      GstClockTime * start, GstClockTime * end)
 {
 	if (gst_base_src_is_live (basesrc)) {
-		GstClockTime timestamp = GST_BUFFER_TIMESTAMP (buffer);
+		GstClockTime timestamp = GST_BUFFER_PTS (buffer);
 
 		if (GST_CLOCK_TIME_IS_VALID (timestamp)) {
 			GstClockTime duration = GST_BUFFER_DURATION (buffer);
@@ -365,8 +359,6 @@ gst_aravis_create (GstPushSrc * push_src, GstBuffer ** buffer)
 	if (arv_buffer == NULL)
 		return GST_FLOW_ERROR;
 
-	*buffer = gst_buffer_new ();
-
 	arv_row_stride = arv_buffer->width * ARV_PIXEL_FORMAT_BIT_PER_PIXEL (arv_buffer->pixel_format) / 8;
 
 	/* Gstreamer requires row stride to be a multiple of 4 */
@@ -384,13 +376,15 @@ gst_aravis_create (GstPushSrc * push_src, GstBuffer ** buffer)
 		for (i = 0; i < arv_buffer->height; i++)
 			memcpy (((char *) data) + i * gst_row_stride, ((char *) arv_buffer->data) + i * arv_row_stride, arv_row_stride);
 
-		GST_BUFFER_DATA (buffer) = data;
-		GST_BUFFER_MALLOCDATA (buffer) = data;
-		GST_BUFFER_SIZE (buffer) = size;
+		*buffer = gst_buffer_new_wrapped (data, size);
 	} else {
-		GST_BUFFER_DATA (*buffer) = arv_buffer->data;
-		GST_BUFFER_MALLOCDATA (*buffer) = NULL;
-		GST_BUFFER_SIZE (*buffer) = arv_buffer->size;
+		*buffer = gst_buffer_new_wrapped_full (0,
+			arv_buffer->data,
+			arv_buffer->size,
+			0,
+			arv_buffer->size,
+			NULL,
+			NULL);
 	}
 
 	if (!gst_base_src_get_do_timestamp(GST_BASE_SRC(push_src))) {
@@ -399,7 +393,7 @@ gst_aravis_create (GstPushSrc * push_src, GstBuffer ** buffer)
 			gst_aravis->last_timestamp = arv_buffer->timestamp_ns;
 		}
 
-		GST_BUFFER_TIMESTAMP (*buffer) = arv_buffer->timestamp_ns - gst_aravis->timestamp_offset;
+		GST_BUFFER_PTS (*buffer) = arv_buffer->timestamp_ns - gst_aravis->timestamp_offset;
 		GST_BUFFER_DURATION (*buffer) = arv_buffer->timestamp_ns - gst_aravis->last_timestamp;
 
 		gst_aravis->last_timestamp = arv_buffer->timestamp_ns;
@@ -407,22 +401,18 @@ gst_aravis_create (GstPushSrc * push_src, GstBuffer ** buffer)
 
 	arv_stream_push_buffer (gst_aravis->stream, arv_buffer);
 
-	gst_buffer_set_caps (*buffer, gst_aravis->fixed_caps);
-
 	return GST_FLOW_OK;
 }
 
-static void
-gst_aravis_fixate_caps (GstPad * pad, GstCaps * caps)
+static GstCaps *
+gst_aravis_fixate_caps (GstBaseSrc * bsrc, GstCaps * caps)
 {
-	GstAravis *gst_aravis = GST_ARAVIS (gst_pad_get_parent_element (pad));
+	GstAravis *gst_aravis = GST_ARAVIS (bsrc);
 	GstStructure *structure;
 	gint width;
 	gint height;
 	double frame_rate;
 
-	g_return_if_fail (GST_IS_ARAVIS (gst_aravis));
-
 	arv_camera_get_region (gst_aravis->camera, NULL, NULL, &width, &height);
 	frame_rate = arv_camera_get_frame_rate (gst_aravis->camera);
 
@@ -434,17 +424,16 @@ gst_aravis_fixate_caps (GstPad * pad, GstCaps * caps)
 
 	GST_LOG_OBJECT (gst_aravis, "Fixate caps");
 
-	g_object_unref (gst_aravis);
+	return GST_BASE_SRC_CLASS(gst_aravis_parent_class)->fixate(bsrc, caps);
 }
 
 static void
-gst_aravis_init (GstAravis *gst_aravis, GstAravisClass *g_class)
+gst_aravis_init (GstAravis *gst_aravis)
 {
 	GstPad *pad = GST_BASE_SRC_PAD (gst_aravis);
 
-	gst_pad_set_fixatecaps_function (pad, gst_aravis_fixate_caps);
-
 	gst_base_src_set_live (GST_BASE_SRC (gst_aravis), TRUE);
+	gst_base_src_set_format (GST_BASE_SRC (gst_aravis), GST_FORMAT_TIME);
 
 	gst_aravis->camera_name = NULL;
 
@@ -456,6 +445,7 @@ gst_aravis_init (GstAravis *gst_aravis, GstAravisClass *g_class)
 	gst_aravis->offset_y = 0;
 	gst_aravis->h_binning = -1;
 	gst_aravis->v_binning = -1;
+        gst_aravis->packet_resend = TRUE;
 	gst_aravis->payload = 0;
 
 	gst_aravis->buffer_timeout_us = GST_ARAVIS_BUFFER_TIMEOUT_DEFAULT;
@@ -492,7 +482,7 @@ gst_aravis_finalize (GObject * object)
 	g_free (gst_aravis->camera_name);
 	gst_aravis->camera_name = NULL;
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gst_aravis_parent_class)->finalize (object);
 }
 
 static void
@@ -546,6 +536,9 @@ gst_aravis_set_property (GObject * object, guint prop_id,
 		case PROP_V_BINNING:
 			gst_aravis->v_binning = g_value_get_int (value);
 			break;
+                case PROP_PACKET_RESEND:
+                        gst_aravis->packet_resend = g_value_get_boolean (value);
+                        break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -589,30 +582,20 @@ gst_aravis_get_property (GObject * object, guint prop_id, GValue * value,
 		case PROP_V_BINNING:
 			g_value_set_int (value, gst_aravis->v_binning);
 			break;
+        	case PROP_PACKET_RESEND:
+                        g_value_set_boolean (value, gst_aravis->packet_resend);
+                        break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
 	}
 }
 
-static void
-gst_aravis_base_init (gpointer g_class)
-{
-	GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
-
-	gst_element_class_set_details_simple (element_class,
-					      "Aravis Video Source",
-					      "Source/Video",
-					      "Aravis based source",
-					      "Emmanuel Pacaud <emmanuel@gnome.org>");
-	gst_element_class_add_pad_template (element_class,
-					    gst_static_pad_template_get (&aravis_src_template));
-}
-
 static void
 gst_aravis_class_init (GstAravisClass * klass)
 {
 	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 	GstBaseSrcClass *gstbasesrc_class = GST_BASE_SRC_CLASS (klass);
 	GstPushSrcClass *gstpushsrc_class = GST_PUSH_SRC_CLASS (klass);
 
@@ -701,10 +684,28 @@ gst_aravis_class_init (GstAravisClass * klass)
 				   1, G_MAXINT, 1,
 				   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_PACKET_RESEND,
+		 g_param_spec_boolean ("packet-resend",
+				       "Packet Resend",
+				       "Request dropped packets to be reissued by the camera",
+				       TRUE,
+				       G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
         GST_DEBUG_CATEGORY_INIT (aravis_debug, "aravissrc", 0, "Aravis interface");
 
+	gst_element_class_set_details_simple (element_class,
+					      "Aravis Video Source",
+					      "Source/Video",
+					      "Aravis based source",
+					      "Emmanuel Pacaud <emmanuel@gnome.org>");
+	gst_element_class_add_pad_template (element_class,
+					    gst_static_pad_template_get (&aravis_src_template));
+
 	gstbasesrc_class->get_caps = gst_aravis_get_caps;
 	gstbasesrc_class->set_caps = gst_aravis_set_caps;
+	gstbasesrc_class->fixate = gst_aravis_fixate_caps;
 	gstbasesrc_class->start = gst_aravis_start;
 	gstbasesrc_class->stop = gst_aravis_stop;
 
@@ -721,7 +722,7 @@ plugin_init (GstPlugin * plugin)
 
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
 		   GST_VERSION_MINOR,
-		   "aravissrc",
+		   aravissrc,
 		   "Aravis Video Source",
 		   plugin_init,
 		   VERSION,
