@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  *
  * Author: Emmanuel Pacaud <emmanuel@gnome.org>
  */
@@ -24,15 +24,13 @@
 #include <gtk/gtk.h>
 #include <gst/gst.h>
 #include <gst/app/gstappsrc.h>
-#include <gst/interfaces/xoverlay.h>
+#include <gst/video/videooverlay.h>
 #include <gdk/gdkx.h>
 #include <arv.h>
 #include <stdlib.h>
 #include <math.h>
 #include <glib/gi18n-lib.h>
-#ifdef ARAVIS_WITH_NOTIFY
 #include <libnotify/notify.h>
-#endif
 
 static char *arv_viewer_option_debug_domains = NULL;
 static gboolean arv_viewer_option_auto_socket_buffer = FALSE;
@@ -89,9 +87,7 @@ typedef struct {
 
 	gulong video_window_xid;
 
-#ifdef ARAVIS_WITH_NOTIFY
 	NotifyNotification *notification;
-#endif
 } ArvViewer;
 
 double
@@ -154,10 +150,11 @@ arv_viewer_new_buffer_cb (ArvStream *stream, ArvViewer *viewer)
 
 	if (arv_buffer->status == ARV_BUFFER_STATUS_SUCCESS) {
 		int arv_row_stride;
-		buffer = gst_buffer_new ();
 
 		arv_row_stride = arv_buffer->width * ARV_PIXEL_FORMAT_BIT_PER_PIXEL (arv_buffer->pixel_format) / 8;
 
+		arv_buffer->timestamp_ns = g_get_real_time () * 1000LL;
+
 		/* Gstreamer requires row stride to be a multiple of 4 */
 		if ((arv_row_stride & 0x3) != 0) {
 			int gst_row_stride;
@@ -173,13 +170,15 @@ arv_viewer_new_buffer_cb (ArvStream *stream, ArvViewer *viewer)
 			for (i = 0; i < arv_buffer->height; i++)
 				memcpy (((char *) data) + i * gst_row_stride, ((char *) arv_buffer->data) + i * arv_row_stride, arv_row_stride);
 
-			GST_BUFFER_DATA (buffer) = data;
-			GST_BUFFER_MALLOCDATA (buffer) = data;
-			GST_BUFFER_SIZE (buffer) = size;
+			buffer = gst_buffer_new_wrapped (data, size);
 		} else {
-			GST_BUFFER_DATA (buffer) = (unsigned char *) arv_buffer->data;
-			GST_BUFFER_MALLOCDATA (buffer) = NULL;
-			GST_BUFFER_SIZE (buffer) = arv_buffer->size;
+			buffer = gst_buffer_new_wrapped_full (GST_MEMORY_FLAG_READONLY,
+				arv_buffer->data,
+				arv_buffer->size,
+				0,
+				arv_buffer->size,
+				NULL,
+				NULL);
 		}
 
 		if (viewer->timestamp_offset == 0) {
@@ -187,7 +186,7 @@ arv_viewer_new_buffer_cb (ArvStream *stream, ArvViewer *viewer)
 			viewer->last_timestamp = arv_buffer->timestamp_ns;
 		}
 
-		GST_BUFFER_TIMESTAMP (buffer) = arv_buffer->timestamp_ns - viewer->timestamp_offset;
+		GST_BUFFER_DTS (buffer) = arv_buffer->timestamp_ns - viewer->timestamp_offset;
 		GST_BUFFER_DURATION (buffer) = arv_buffer->timestamp_ns - viewer->last_timestamp;
 
 		gst_app_src_push_buffer (GST_APP_SRC (viewer->appsrc), buffer);
@@ -432,7 +431,6 @@ arv_viewer_snapshot_cb (GtkButton *button, ArvViewer *viewer)
 				 "Aravis", filename, NULL);
 	g_file_set_contents (path, viewer->last_buffer->data, viewer->last_buffer->size, NULL);
 
-#ifdef ARAVIS_WITH_NOTIFY
 	if (viewer->notification) {
 		notify_notification_update (viewer->notification,
 					    "Snapshot saved to Image folder",
@@ -440,7 +438,6 @@ arv_viewer_snapshot_cb (GtkButton *button, ArvViewer *viewer)
 					    "gtk-save");
 		notify_notification_show (viewer->notification, NULL);
 	}
-#endif
 
 	g_free (path);
 	g_free (filename);
@@ -491,21 +488,14 @@ bus_sync_handler (GstBus *bus, GstMessage *message, gpointer user_data)
 {
 	ArvViewer *viewer = user_data;
 
-	if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
-		return GST_BUS_PASS;
-	if (!gst_structure_has_name (message->structure, "prepare-xwindow-id"))
+	if (!gst_is_video_overlay_prepare_window_handle_message(message))
 		return GST_BUS_PASS;
 
 	if (viewer->video_window_xid != 0) {
-		GstXOverlay *xoverlay;
+		GstVideoOverlay *videooverlay;
 
-		xoverlay = GST_X_OVERLAY (GST_MESSAGE_SRC (message));
-		gst_x_overlay_set_window_handle (xoverlay, viewer->video_window_xid);
-
-		if (g_strcmp0 (G_OBJECT_TYPE_NAME (xoverlay), "GstXvImageSink"))
-			g_object_set (xoverlay, "draw-borders", TRUE, NULL);
-
-		g_object_set (xoverlay, "force-aspect-ratio", TRUE, "sync", FALSE, NULL);
+		videooverlay = GST_VIDEO_OVERLAY (GST_MESSAGE_SRC (message));
+		gst_video_overlay_set_window_handle (videooverlay, viewer->video_window_xid);
 	} else {
 		g_warning ("Should have obtained video_window_xid by now!");
 	}
@@ -521,7 +511,7 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 	GtkTreeIter iter;
 	GtkTreeModel *list_store;
 	GstCaps *caps;
-	GstElement *ffmpegcolorspace;
+	GstElement *videoconvert;
 	GstElement *videosink;
 	GstBus *bus;
 	ArvPixelFormat pixel_format;
@@ -654,23 +644,23 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 	viewer->pipeline = gst_pipeline_new ("pipeline");
 
 	viewer->appsrc = gst_element_factory_make ("appsrc", NULL);
-	ffmpegcolorspace = gst_element_factory_make ("ffmpegcolorspace", NULL);
+	videoconvert = gst_element_factory_make ("videoconvert", NULL);
 	viewer->transform = gst_element_factory_make ("videoflip", NULL);
 	videosink = gst_element_factory_make ("autovideosink", NULL);
 
-	if (g_str_has_prefix (caps_string, "video/x-raw-bayer")) {
+	if (g_str_has_prefix (caps_string, "video/x-bayer")) {
 		GstElement *bayer2rgb;
 
 		bayer2rgb = gst_element_factory_make ("bayer2rgb", NULL);
 
 		gst_bin_add_many (GST_BIN (viewer->pipeline), viewer->appsrc, bayer2rgb,
-				  ffmpegcolorspace, viewer->transform, videosink, NULL);
+				  videoconvert, viewer->transform, videosink, NULL);
 		gst_element_link_many (viewer->appsrc, bayer2rgb,
-				       ffmpegcolorspace, viewer->transform, videosink, NULL);
+				       videoconvert, viewer->transform, videosink, NULL);
 	} else {
 		gst_bin_add_many (GST_BIN (viewer->pipeline), viewer->appsrc,
-				  ffmpegcolorspace, viewer->transform, videosink, NULL);
-		gst_element_link_many (viewer->appsrc, ffmpegcolorspace,
+				  videoconvert, viewer->transform, videosink, NULL);
+		gst_element_link_many (viewer->appsrc, videoconvert,
 				       viewer->transform, videosink, NULL);
 	}
 
@@ -683,8 +673,10 @@ arv_viewer_select_camera_cb (GtkComboBox *combo_box, ArvViewer *viewer)
 	gst_app_src_set_caps (GST_APP_SRC (viewer->appsrc), caps);
 	gst_caps_unref (caps);
 
+	g_object_set(G_OBJECT (viewer->appsrc), "format", GST_FORMAT_TIME, NULL);
+
 	bus = gst_pipeline_get_bus (GST_PIPELINE (viewer->pipeline));
-	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler, viewer);
+	gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler, viewer, NULL);
 	gst_object_unref (bus);
 
 	gst_element_set_state (viewer->pipeline, GST_STATE_PLAYING);
@@ -697,10 +689,8 @@ arv_viewer_free (ArvViewer *viewer)
 {
 	g_return_if_fail (viewer != NULL);
 
-#ifdef ARAVIS_WITH_NOTIFY
 	if (viewer->notification)
 		g_object_unref (viewer->notification);
-#endif
 
 	if (viewer->exposure_update_event > 0)
 		g_source_remove (viewer->exposure_update_event);
@@ -736,13 +726,7 @@ arv_viewer_new (void)
 
 	builder = gtk_builder_new ();
 
-	ui_filename = g_build_filename (ARAVIS_DATA_DIR,
-#if GTK_CHECK_VERSION(3,0,0)
-					"arv-viewer-3.ui",
-#else
-					"arv-viewer-2.ui",
-#endif
-					NULL);
+	ui_filename = g_build_filename (ARAVIS_DATA_DIR, "arv-viewer.ui", NULL);
 
 	if (!gtk_builder_add_from_file (builder, ui_filename, NULL))
 		g_error ("The user interface file is missing ('%s')", ui_filename);
@@ -801,9 +785,7 @@ arv_viewer_new (void)
 	viewer->auto_gain_clicked = g_signal_connect (viewer->auto_gain_toggle, "clicked",
 						      G_CALLBACK (arv_viewer_auto_gain_cb), viewer);
 
-#ifdef ARAVIS_WITH_NOTIFY
 	viewer->notification = notify_notification_new (NULL, NULL, NULL);
-#endif
 
 	return viewer;
 }
@@ -842,13 +824,13 @@ _gstreamer_plugin_check (void)
 
 	static char *plugins[] = {
 		"appsrc",
-		"ffmpegcolorspace",
+		"videoconvert",
 		"videoflip",
 		"autovideosink",
 		"bayer2rgb"
 	};
 
-	registry = gst_registry_get_default ();
+	registry = gst_registry_get ();
 
 	for (i = 0; i < G_N_ELEMENTS (plugins); i++) {
 		GstPluginFeature *feature;
@@ -908,9 +890,7 @@ main (int argc,char *argv[])
 
 	arv_enable_interface ("Fake");
 
-#ifdef ARAVIS_WITH_NOTIFY
 	notify_init ("Aravis Viewer");
-#endif
 
 	viewer = arv_viewer_new ();
 
@@ -919,9 +899,7 @@ main (int argc,char *argv[])
 
 	gtk_main ();
 
-#ifdef ARAVIS_WITH_NOTIFY
 	notify_uninit ();
-#endif
 
 	/* For debug purpose only */
 	arv_shutdown ();
